@@ -21,7 +21,7 @@ use warnings;
 no warnings qw(recursion);
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '2.068000';
+$VERSION        = '2.070000';
 $STRING_VERSION = $VERSION;
 ## no critic(BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -574,22 +574,26 @@ sub Marpa::R2::Scanless::ASF::new {
         {with the Marpa::R2::Scanless::ASF::new method}
     ) if not defined $force and not defined $default_blessing;
 
+    my $recce     = $slr->[Marpa::R2::Inner::Scanless::R::THICK_G1_RECCE];
+
+    if ( defined $recce->[Marpa::R2::Internal::Recognizer::TREE_MODE] ) {
+        # If we already in ASF mode, or are in valuation mode, we cannot create an ASF
+        Marpa::R2::exception(
+            "An attempt was made to create an ASF for a SLIF recognizer already in use\n",
+            "   The recognizer must be reset first\n",
+            '  The current SLIF recognizer mode is "',
+            $recce->[Marpa::R2::Internal::Recognizer::TREE_MODE],
+            qq{"\n}
+        );
+    }
+    $recce->[Marpa::R2::Internal::Recognizer::TREE_MODE] = 'forest';
+
     my $slg       = $slr->[Marpa::R2::Inner::Scanless::R::GRAMMAR];
     my $thin_slr  = $slr->[Marpa::R2::Inner::Scanless::R::C];
-    my $recce     = $slr->[Marpa::R2::Inner::Scanless::R::THICK_G1_RECCE];
     my $grammar   = $recce->[Marpa::R2::Internal::Recognizer::GRAMMAR];
     my $grammar_c = $grammar->[Marpa::R2::Internal::Grammar::C];
     my $recce_c   = $recce->[Marpa::R2::Internal::Recognizer::C];
 
-    if ( $recce->[Marpa::R2::Internal::Recognizer::T_C] ) {
-
-        # If there is a tree we are in valuation mode, and cannot create an ASF
-        Marpa::R2::exception(
-            'An attempt was made to create an ASF for a SLIF recognizer in value mode',
-            '   The recognizer must be reset first'
-        );
-    } ## end if ( $recce->[Marpa::R2::Internal::Recognizer::T_C] )
-
     my $bocage = $recce->[Marpa::R2::Internal::Recognizer::B_C];
     if ( not $bocage ) {
         my $grammar   = $recce->[Marpa::R2::Internal::Recognizer::GRAMMAR];
@@ -602,10 +606,7 @@ sub Marpa::R2::Scanless::ASF::new {
         return if not defined $bocage;
     } ## end if ( not $bocage )
 
-    my $rule_resolutions =
-        $recce->[Marpa::R2::Internal::Recognizer::RULE_RESOLUTIONS]
-        // Marpa::R2::Internal::Recognizer::semantics_set( $recce,
-        Marpa::R2::Internal::Recognizer::default_semantics($recce) );
+    my $rule_resolutions = Marpa::R2::Internal::Recognizer::semantics_set( $recce);
 
     my $default_blessing_by_rule_id   = $rule_resolutions->{blessing};
     my $default_blessing_by_lexeme_id = $rule_resolutions->{blessing_by_lexeme};
@@ -614,7 +615,7 @@ sub Marpa::R2::Scanless::ASF::new {
     my $highest_rule_id = $grammar_c->highest_rule_id();
     RULE: for ( my $rule_id = 0; $rule_id <= $highest_rule_id; $rule_id++ ) {
         my $lhs_id = $grammar_c->rule_lhs($rule_id);
-        my $name   = $grammar->symbol_dsl_name($lhs_id);
+        my $name   = $grammar->symbol_name($lhs_id);
         if ( defined $force ) {
             $rule_blessing[$rule_id] = join q{::}, $force,
                 normalize_asf_blessing($name);
@@ -635,7 +636,7 @@ sub Marpa::R2::Scanless::ASF::new {
     my $highest_symbol_id = $grammar_c->highest_symbol_id();
     SYMBOL: for ( my $symbol_id = 0; $symbol_id <= $highest_symbol_id; $symbol_id++ )
     {
-        my $name = $grammar->symbol_dsl_name($symbol_id);
+        my $name = $grammar->symbol_name($symbol_id);
         if ( defined $force ) {
             $symbol_blessing[$symbol_id] = join q{::}, $force,
                 normalize_asf_blessing($name);
